[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Standard Datafeed client library for Python

[![API Version](https://img.shields.io/badge/api-v2.0.0-blue)](https://developer.factset.com/api-catalog/standard-datafeed-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.StandardDatafeed/1.0.3)](https://pypi.org/project/fds.sdk.StandardDatafeed/v/1.0.3)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

[The Standard DataFeed (SDF) Download API](https://www.factset.com/marketplace/catalog/product/factset-standard-datafeed-download-api) 
provides an alternative method for users to request and retrieve SDF packages (schemas & bundles). 
This service is not a direct replacement and does not have 100% feature parity with the Loader Application. 
This API provides an alternative for users who are unable to utilize the Loader application due to the following reasons:

* Inability to install 3rd party executables due to Corporate Security policies.

* Inability to utilize the Loader application due to limitations or restrictions with the environment used to consume Standard Datafeed.

* Clients who are utilizing existing delivery methods like FTP, who may want to use a more secure & modern solution.

This API allows users to retrieve

* Both SDF and the QFL (Quant Factor Library (Factor Family & Factor Groups)) packages they have subscriptions for, with data available since January 1, 1995.

Additional parameters are available to filter requests to get the exact files users are looking for.

QFL data is delivered through Content API & Bulk Data API (SDF API).

* Content API: Provides direct access to FactSet-hosted QFL data. Suitable for interactive, ad hoc QFL requests. Constraints on large extracts. Costs are based on consumption, i.e. more calls can result in more costs.

* Bulk Data API: Provides access to download locations of zip files for client download. Suitable for production processes within a client environment. Cost is based on the use case and fixed unless scope changes (same as other SDFs).

Please find all the content-related comprehensive documentation [here](https://my.apps.factset.com/oa/pages/15222).


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 2.0.0
- SDK version: 1.0.3
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.StandardDatafeed==1.0.3
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.StandardDatafeed==1.0.3
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.StandardDatafeed
from fds.sdk.StandardDatafeed.api import sdf_and_qfl_content_library_api
from fds.sdk.StandardDatafeed.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.StandardDatafeed.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.StandardDatafeed.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.StandardDatafeed.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = sdf_and_qfl_content_library_api.SDFAndQFLContentLibraryApi(api_client)
    schema = "fgp_v1" # str | Name of the schema. (optional)
    bundle = "fgp_global_prices_am_v1" # str | Name of the bundle. (optional)
    type = "delta" # str | Type of the file.  Note: - Full files are snapshots of the bundle capturing the most recent version of the bundle generated every weekend. When requesting 'Full' files, ensure that the date range includes weekend dates. - Delta files include the incremental changes (inserts, updates, deletes), since the last file and have incremental sequence numbers.    (optional) if omitted the server will use the default value of "delta"
    start_date = "2023-01-01" # str | The earliest date of the feed file the API should fetch based on the file timestamp. Consider the following points:   - Dates provided in `startDate` and `endDate` along with `schema` parameter: The returned dataset is limited to a maximum of latest 30 days' worth of records. - Format: Should be absolute (YYYY-MM-DD).  (optional)
    start_date_relative = 1 # int | The earliest date of the feed file the API should fetch based on the file timestamp. Consider the following points:  - Dates provided in `startDate` and `endDate` along with `schema` parameter: The returned dataset is limited to a maximum of latest 30 days' worth of records. - Format: Specify the date using a relative term as an integer: '0' for today, '-1' for yesterday, '-2' for two days ago, and so forth. Negative values are used to represent past dates.  *Note:* - *Either `startDate` or `startDateRelative` should be used, but not both.* - *If both `startDate` and `startDateRelative` are provided in the same request, the API will return an error.* - *If users provide future dates in requests for `startDate` or `startDateRelative`, the API will not return any data.*  (optional)
    end_date = "2023-01-28" # str | The latest date of the feed file the API should fetch for based on the file timestamp.  - Format: Should be absolute - YYYY-MM-DD.  (optional)
    end_date_relative = 1 # int | The latest date of the feed file the API should fetch for based on the file timestamp.  - Format: Specify the date using a relative term as an integer: '0' for today, '-1' for yesterday, '-2' for two days ago, and so forth. Negative values are used to represent past dates.  *Note:* - *Either `endDate` or `endDateRelative` should be used, but not both.* - *If both `endDate` and `endDateRelative` are provided in the same request, the API will return an error.* - *If users provide future dates in requests for `endDate` or `endDateRelative`, the API will not return any data.*  (optional)
    pagination_limit = 20 # int | Specifies the number of results to return per page. (optional) if omitted the server will use the default value of 20
    pagination_offset = 0 # int | Specifies the starting point for pagination. This parameter is used to identify the beginning of next set of results. (optional) if omitted the server will use the default value of 0
    sort = ["-startDate"] # [str] | Enables sorting data in ascending or descending chronological order based on startDate.  (optional) if omitted the server will use the default value of ["-startDate"]

    try:
        # Returns delta & full files for the schemas.
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_list_files(schema=schema, bundle=bundle, type=type, start_date=start_date, start_date_relative=start_date_relative, end_date=end_date, end_date_relative=end_date_relative, pagination_limit=pagination_limit, pagination_offset=pagination_offset, sort=sort)

        pprint(api_response)
    except fds.sdk.StandardDatafeed.ApiException as e:
        print("Exception when calling SDFAndQFLContentLibraryApi->get_list_files: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Returns delta & full files for the schemas.
    #     api_response, http_status_code, response_headers = api_instance.get_list_files_with_http_info(schema=schema, bundle=bundle, type=type, start_date=start_date, start_date_relative=start_date_relative, end_date=end_date, end_date_relative=end_date_relative, pagination_limit=pagination_limit, pagination_offset=pagination_offset, sort=sort)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.StandardDatafeed.ApiException as e:
    #     print("Exception when calling SDFAndQFLContentLibraryApi->get_list_files: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Returns delta & full files for the schemas.
    #     async_result = api_instance.get_list_files_async(schema=schema, bundle=bundle, type=type, start_date=start_date, start_date_relative=start_date_relative, end_date=end_date, end_date_relative=end_date_relative, pagination_limit=pagination_limit, pagination_offset=pagination_offset, sort=sort)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.StandardDatafeed.ApiException as e:
    #     print("Exception when calling SDFAndQFLContentLibraryApi->get_list_files: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Returns delta & full files for the schemas.
    #     async_result = api_instance.get_list_files_with_http_info_async(schema=schema, bundle=bundle, type=type, start_date=start_date, start_date_relative=start_date_relative, end_date=end_date, end_date_relative=end_date_relative, pagination_limit=pagination_limit, pagination_offset=pagination_offset, sort=sort)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.StandardDatafeed.ApiException as e:
    #     print("Exception when calling SDFAndQFLContentLibraryApi->get_list_files: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.StandardDatafeed

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.StandardDatafeed.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.StandardDatafeed

configuration = fds.sdk.StandardDatafeed.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.StandardDatafeed

configuration = fds.sdk.StandardDatafeed.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.StandardDatafeed

configuration = fds.sdk.StandardDatafeed.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/bulk-documents/sdf/v2*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*SDFAndQFLContentLibraryApi* | [**get_list_files**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/SDFAndQFLContentLibraryApi.md#get_list_files) | **GET** /list-files | Returns delta &amp; full files for the schemas.
*SDFAndQFLContentLibraryApi* | [**gethistorical_files**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/SDFAndQFLContentLibraryApi.md#gethistorical_files) | **GET** /historical-files | Returns full historic data of specified schema and bundle.
*SchemaApi* | [**get_list_schemaswithoutwithoout_required_parameters**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/SchemaApi.md#get_list_schemaswithoutwithoout_required_parameters) | **GET** /list-schemas | List of Standard DataFeed (SDF) schemas.
*SchemaApi* | [**get_schema_details**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/SchemaApi.md#get_schema_details) | **GET** /schema-details | Schema Details.


## Documentation For Models

 - [ErrorExample](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/ErrorExample.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/ErrorResponse.md)
 - [HistoricalFile](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/HistoricalFile.md)
 - [HistoricalFileObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/HistoricalFileObject.md)
 - [HistoricalFileObjectObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/HistoricalFileObjectObject.md)
 - [ListFile](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/ListFile.md)
 - [ListFileObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/ListFileObject.md)
 - [ListFileObjectObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/ListFileObjectObject.md)
 - [ListSchema](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/ListSchema.md)
 - [ListSchemaObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/ListSchemaObject.md)
 - [Meta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/Meta.md)
 - [Pagination](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/Pagination.md)
 - [SchemaDetail](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/SchemaDetail.md)
 - [SchemaDetailObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/StandardDatafeed/v2/docs/SchemaDetailObject.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.StandardDatafeed.apis and fds.sdk.StandardDatafeed.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.StandardDatafeed.api.default_api import DefaultApi`
- `from fds.sdk.StandardDatafeed.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.StandardDatafeed
from fds.sdk.StandardDatafeed.apis import *
from fds.sdk.StandardDatafeed.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

